@@ -2,7 +2,7 @@ package Minion::Backend::Mango;
 use Mojo::Base 'Minion::Backend';
 
 use Mango;
-use Mango::BSON 'bson_time';
+use Mango::BSON qw(bson_oid bson_time);
 use Scalar::Util 'weaken';
 use Sys::Hostname 'hostname';
 
@@ -13,7 +13,7 @@ has workers =>
   sub { $_[0]->mango->db->collection($_[0]->prefix . '.workers') };
 
 sub dequeue {
-  my ($self, $oid) = @_;
+  my ($self, $id) = @_;
 
   my $doc = {
     query => {
@@ -23,8 +23,10 @@ sub dequeue {
     },
     fields => {args     => 1, task => 1},
     sort   => {priority => -1},
-    update =>
-      {'$set' => {started => bson_time, state => 'active', worker => $oid}},
+    update => {
+      '$set' =>
+        {started => bson_time, state => 'active', worker => bson_oid($id)}
+    },
     new => 1
   };
 
@@ -58,7 +60,7 @@ sub fail_job { shift->_update(1, @_) }
 
 sub finish_job { shift->_update(0, @_) }
 
-sub job_info { $_[0]->_job_info($_[0]->jobs->find_one($_[1])) }
+sub job_info { $_[0]->_job_info($_[0]->jobs->find_one(bson_oid($_[1]))) }
 
 sub list_jobs    { shift->_list('jobs',    'state', @_) }
 sub list_workers { shift->_list('workers', 'pid',   @_) }
@@ -70,8 +72,9 @@ sub register_worker {
 }
 
 sub remove_job {
-  my ($self, $oid) = @_;
-  my $doc = {_id => $oid, state => {'$in' => [qw(failed finished inactive)]}};
+  my ($self, $id) = @_;
+  my $doc = {_id => bson_oid($id),
+    state => {'$in' => [qw(failed finished inactive)]}};
   return !!$self->jobs->remove($doc)->{n};
 }
 
@@ -97,9 +100,10 @@ sub repair {
 sub reset { $_->options && $_->drop for $_[0]->workers, $_[0]->jobs }
 
 sub restart_job {
-  my ($self, $oid) = @_;
+  my ($self, $id) = @_;
 
-  my $query = {_id => $oid, state => {'$in' => [qw(failed finished)]}};
+  my $query
+    = {_id => bson_oid($id), state => {'$in' => [qw(failed finished)]}};
   my $update = {
     '$inc' => {restarts  => 1},
     '$set' => {restarted => bson_time, state => 'inactive'},
@@ -122,9 +126,11 @@ sub stats {
   return $stats;
 }
 
-sub unregister_worker { shift->workers->remove({_id => shift}) }
+sub unregister_worker { shift->workers->remove({_id => bson_oid(shift)}) }
 
-sub worker_info { $_[0]->_worker_info($_[0]->workers->find_one($_[1])) }
+sub worker_info {
+  $_[0]->_worker_info($_[0]->workers->find_one(bson_oid($_[1])));
+}
 
 sub _job_info {
   my $self = shift;
@@ -136,7 +142,7 @@ sub _job_info {
     delayed   => $job->{delayed} ? $job->{delayed}->to_epoch : undef,
     error     => $job->{error},
     finished  => $job->{finished} ? $job->{finished}->to_epoch : undef,
-    id        => $job->{_id},
+    id        => "$job->{_id}",
     priority  => $job->{priority},
     restarted => $job->{restarted} ? $job->{restarted}->to_epoch : undef,
     restarts => $job->{restarts} // 0,
@@ -157,11 +163,11 @@ sub _list {
 }
 
 sub _update {
-  my ($self, $fail, $oid, $err) = @_;
+  my ($self, $fail, $id, $err) = @_;
 
   my $update = {finished => bson_time, state => $fail ? 'failed' : 'finished'};
   $update->{error} = $err if $fail;
-  my $query = {_id => $oid, state => 'active'};
+  my $query = {_id => bson_oid($id), state => 'active'};
   return !!$self->jobs->update($query, {'$set' => $update})->{n};
 }
 
@@ -173,7 +179,7 @@ sub _worker_info {
     = $self->jobs->find({state => 'active', worker => $worker->{_id}});
   return {
     host    => $worker->{host},
-    id      => $worker->{_id},
+    id      => "$worker->{_id}",
     jobs    => [map { $_->{_id} } @{$cursor->all}],
     pid     => $worker->{pid},
     started => $worker->{started}->to_epoch
@@ -262,7 +268,7 @@ perform operation non-blocking.
 
 =head2 fail_job
 
-  my $bool = $backend->fail_job;
+  my $bool = $backend->fail_job($job_id);
   my $bool = $backend->fail_job($job_id, 'Something went wrong!');
 
 Transition from C<active> to C<failed> state.
@@ -323,7 +329,7 @@ Reset job queue.
 
 =head2 restart_job
 
-  my $bool = $backend->restart_job;
+  my $bool = $backend->restart_job($job_id);
 
 Transition from C<failed> or C<finished> state back to C<inactive>.
 
